'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DepTrackingStorageCache = exports.InStorageCache = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _apolloCacheInmemory = require('apollo-cache-inmemory');

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaults = {
  normalize: JSON.stringify,
  denormalize: JSON.parse
};

var validStorage = function validStorage(storage) {
  return Boolean(storage && storage.getItem && storage.setItem && storage.removeItem && storage.clear);
};

var InStorageCacheError = function (_Error) {
  _inherits(InStorageCacheError, _Error);

  function InStorageCacheError(message) {
    var _ref;

    _classCallCheck(this, InStorageCacheError);

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    return _possibleConstructorReturn(this, (_ref = InStorageCacheError.__proto__ || Object.getPrototypeOf(InStorageCacheError)).call.apply(_ref, [this, '[InStorageCacheError] ' + message].concat(args)));
  }

  return InStorageCacheError;
}(Error);

var InStorageCache = function (_InMemoryCache) {
  _inherits(InStorageCache, _InMemoryCache);

  /**
   * @property {(Object|Function)} storage - The Storage to use.
   *
   * @see https://www.w3.org/TR/webstorage/#storage
   */


  /**
   * @property {Function} [normalize] - Normalization callback. Executed
   * prior to storing a resource.
   */


  /**
   * @property {Function} [denormalize] - Denormalization callback. Executed
   * after retrieving a cached resource from the storage.
   */
  function InStorageCache() {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var storage = _ref2.storage,
        _ref2$normalize = _ref2.normalize,
        normalize = _ref2$normalize === undefined ? DepTrackingStorageCache.normalize : _ref2$normalize,
        _ref2$denormalize = _ref2.denormalize,
        denormalize = _ref2$denormalize === undefined ? DepTrackingStorageCache.denormalize : _ref2$denormalize,
        _ref2$shouldPersist = _ref2.shouldPersist,
        shouldPersist = _ref2$shouldPersist === undefined ? function () {
      return true;
    } : _ref2$shouldPersist,
        config = _objectWithoutProperties(_ref2, ['storage', 'normalize', 'denormalize', 'shouldPersist']);

    _classCallCheck(this, InStorageCache);

    var _this2 = _possibleConstructorReturn(this, (InStorageCache.__proto__ || Object.getPrototypeOf(InStorageCache)).call(this, config));

    if (!storage) {
      throw new InStorageCacheError('You must provide a storage to use');
    }

    if (!validStorage(storage)) {
      throw new InStorageCacheError('You must provide a valid storage to use');
    }

    _this2.persistence = {
      storage: storage,
      normalize: normalize,
      denormalize: denormalize,
      shouldPersist: shouldPersist
    };

    _this2.data = new DepTrackingStorageCache(null, _this2.persistence);
    return _this2;
  }

  /**
   * @property {Function} [shouldPersist] - Callback to determine if a given
   * data should be cached.
   */


  return InStorageCache;
}(_apolloCacheInmemory.InMemoryCache);

var DepTrackingStorageCache = function () {
  _createClass(DepTrackingStorageCache, null, [{
    key: 'iterate',

    /**
     * Iterates each key of the storage and execute the callback on it.
     *
     * @param {Object} storage The storage instance.
     * @param {Function} callback The iteration callback.
     */
    value: function iterate(storage, callback) {
      for (var i = 0; i < storage.length; ++i) {
        callback(storage.key(i));
      }
    }

    /**
     * Creates a plain object from all the storage's persisted data.
     *
     * @param {Object} storage The storage instance.
     * @param {Function} denormalize Method of denormalizing the retrieved resource.
     */

  }, {
    key: 'toObject',
    value: function toObject(storage) {
      var denormalize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaults.denormalize;

      var object = {};

      DepTrackingStorageCache.iterate(storage, function (key) {
        object[key] = denormalize(storage.getItem(key));
      });

      return object;
    }
  }]);

  function DepTrackingStorageCache() {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var persistence = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, DepTrackingStorageCache);

    if (!persistence.storage) {
      throw new InStorageCacheError('You must provide a persistence.storage to use');
    }

    if (!validStorage(persistence.storage)) {
      throw new InStorageCacheError('You must provide a valid persistence.storage to use');
    }

    if (typeof persistence.normalize !== 'function') {
      throw new InStorageCacheError('You must provide a persistence.normalize function');
    }

    if (typeof persistence.denormalize !== 'function') {
      throw new InStorageCacheError('You must provide a persistence.denormalize function');
    }

    if (typeof persistence.shouldPersist !== 'function') {
      throw new InStorageCacheError('You must provide a persistence.shouldPersist function');
    }

    this.persistence = persistence;
    this.data = _extends({}, data);
  }

  _createClass(DepTrackingStorageCache, [{
    key: 'toObject',
    value: function toObject() {
      var persisted = DepTrackingStorageCache.toObject(this.persistence.storage, this.persistence.denormalize);

      return _extends({}, persisted, this.data);
    }
  }, {
    key: 'get',
    value: function get(dataId) {
      if (!this.data[dataId] && this.persistence.shouldPersist('get', dataId)) {
        this.data[dataId] = this.persistence.denormalize(this.persistence.storage.getItem(dataId));
      }

      return this.data[dataId];
    }
  }, {
    key: 'set',
    value: function set(dataId, value) {
      if (this.persistence.shouldPersist('set', dataId, value)) {
        this.persistence.storage.setItem(dataId, this.persistence.normalize(value));
      }

      this.data[dataId] = value;
    }
  }, {
    key: 'delete',
    value: function _delete(dataId) {
      if (this.persistence.shouldPersist('delete', dataId)) {
        this.persistence.storage.removeItem(dataId);
      }

      this.data[dataId] = undefined;
    }
  }, {
    key: 'clear',
    value: function clear() {
      this.persistence.storage.clear();
      this.data = {};
    }
  }, {
    key: 'replace',
    value: function replace(newData) {
      this.data = {};

      for (var dataId in newData) {
        this.set(dataId, newData[dataId]);
      }
    }
  }]);

  return DepTrackingStorageCache;
}();

DepTrackingStorageCache.normalize = defaults.normalize;
DepTrackingStorageCache.denormalize = defaults.denormalize;
exports.InStorageCache = InStorageCache;
exports.DepTrackingStorageCache = DepTrackingStorageCache;